@@ -161,7 +161,7 @@ def get_sites(api_inputs: ApiInputs, include_tag_groups: Union[list, bool] = Fal
         If False, no tag groups are included. If True, all tag groups will be returned. Else, if list, the Tag Groups
         in the list are retrieved as columns.
     sql_where_clause : str, default = None
-        Optional `WHERE` clause in SQL syntax.
+        Optional `WHERE` clause in SQL syntax. Use field names only and do not include the "WHERE".
     top_count: int, default = None
         For use during testing to limit the number of records returned.
 
@@ -184,6 +184,11 @@ def get_sites(api_inputs: ApiInputs, include_tag_groups: Union[list, bool] = Fal
         tag_groups = []
         tags_mode = include_tag_groups
 
+    if sql_where_clause is not None:
+        if sql_where_clause.startswith('WHERE') or sql_where_clause.startswith('where'):
+            sql_where_clause = sql_where_clause.removeprefix('WHERE')
+            sql_where_clause = sql_where_clause.removeprefix('where')
+
     payload = {
         "tagsMode": tags_mode,
         "includeTagColumns": tag_groups,
