@@ -52,6 +52,6 @@ retrieving details of existing tasks on the Switch Automation platform are also
 
 """
 
-from .pipeline import IntegrationTask, QueueTask, LogicModuleTask, AnalyticsTask, Automation
+from .pipeline import IntegrationTask, QueueTask, LogicModuleTask, AnalyticsTask, Automation, logger
 
-__all__ = ['IntegrationTask', 'QueueTask', 'LogicModuleTask', 'AnalyticsTask', 'Automation']
+__all__ = ['IntegrationTask', 'QueueTask', 'LogicModuleTask', 'AnalyticsTask', 'Automation', 'logger']
