@@ -60,12 +60,11 @@ import inspect
 import uuid
 import logging
 import datetime
-# import base64
-# import asyncio
+import json
 from typing import List
 from abc import ABC, abstractmethod
 from azure.servicebus import ServiceBusClient, ServiceBusReceiveMode  # , ServiceBusMessage
-from .._utils._utils import _is_valid_regex, ApiInputs, generate_password  # , ValidatedSetterProperty
+from .._utils._utils import _is_valid_regex, ApiInputs, generate_password, _column_name_cap, DataFeedFileProcessOutput
 from .._utils._constants import api_prefix, argus_prefix, EXPECTED_DELIVERY, DEPLOY_TYPE, MAPPING_ENTITIES, QUEUE_NAME
 from .._utils._platform import _get_connection_string
 
@@ -102,6 +101,7 @@ class Task(ABC):
         The version of the task.
 
     """
+
     @property
     @abstractmethod
     def id(self) -> uuid.UUID:
@@ -322,7 +322,7 @@ class Automation:
 
         logger.info("Running queue for  %s (%s) using queue name: %s", str(type(task).__name__), str(task.id),
                     str(task.queue_name))
-        
+
         task.start(api_inputs)
 
         message_count = 0
@@ -342,7 +342,7 @@ class Automation:
                         break
 
                     task.process_queue(api_inputs, messages)
-                    
+
                     if consume_all_messages == False:
                         break
         logger.info('Total messages consumed: %s', str(message_count))
@@ -397,6 +397,8 @@ class Automation:
             return response_status, pandas.DataFrame()
 
         df = pandas.read_json(response.text, typ="Series")
+        df.columns = _column_name_cap(df.columns)
+
         return df
 
     @staticmethod
@@ -459,7 +461,7 @@ class Automation:
             "name": type(task).__name__,
             "specification": task.description,
             "mappingEntities": task.mapping_entities,
-            "scriptCode": Automation.get_task_code(task),
+            "scriptCode": Automation._get_task_code(task),
             "baseClass": base_class
         }
 
@@ -476,6 +478,8 @@ class Automation:
             return response_status, pandas.DataFrame()
 
         df = pandas.read_json(response.text)
+        df.columns = _column_name_cap(df.columns)
+
         return df
 
     @staticmethod
@@ -519,6 +523,8 @@ class Automation:
 
         df = pandas.read_json(response.text)
         df = df.drop(columns=['driverId'])
+        df.columns = _column_name_cap(df.columns)
+
         return df
 
     @staticmethod
@@ -680,12 +686,14 @@ class Automation:
             return response_status, pandas.DataFrame()
 
         df = pandas.read_json(response.text, typ='Series')
+        df.columns = _column_name_cap(df.columns)
+
         return df
 
     @staticmethod
     def deploy_as_upload_data_feed(task, api_inputs: ApiInputs, queue_name: QUEUE_NAME = "task",
                                    expected_delivery: EXPECTED_DELIVERY = "15min"):
-        """Deploy the custom driver as a Upload Datafeed ie Customer sends data via a Rest ENd Point
+        """Deploy the custom driver as a REST API end point Datafeed.
 
         Parameters
         ----------
@@ -701,7 +709,7 @@ class Automation:
         Returns
         -------
         df : pandas.DataFrame
-            Dataframe containing the details of the deployed ftp data feed.
+            Dataframe containing the details of the deployed https endpoint data feed.
 
         """
 
@@ -744,6 +752,8 @@ class Automation:
             return response_status, pandas.DataFrame()
 
         df = pandas.read_json(response.text)
+        df.columns = _column_name_cap(df.columns)
+
         return df
 
     @staticmethod
@@ -830,6 +840,8 @@ class Automation:
             return response_status, pandas.DataFrame()
 
         df = pandas.read_json(response.text, typ='Series')
+        df.columns = _column_name_cap(df.columns)
+
         return df
 
     @staticmethod
@@ -870,7 +882,8 @@ class Automation:
         # payload = {}
 
         url = api_prefix + api_inputs.datacentre + "/" + api_inputs.api_project_id + "/Automation/CancelTask/" + \
-            deploy_type + "/" + task.id
+              deploy_type + "/" + task.id
+
         response = requests.get(url, headers=headers)
         response_status = '{} {}'.format(response.status_code, response.reason)
         if response.status_code != 200:
@@ -882,6 +895,8 @@ class Automation:
             return response_status, pandas.DataFrame()
 
         df = pandas.read_json(response.text, typ='Series')
+        df.columns = _column_name_cap(df.columns)
+
         return df
 
     @staticmethod
@@ -914,7 +929,7 @@ class Automation:
         }
 
         url = api_prefix + api_inputs.datacentre + "/" + api_inputs.api_project_id + "/Automation/ListDeployments/" + \
-            search_name_pattern
+              search_name_pattern
 
         response = requests.request("GET", url, timeout=20, headers=headers)
         response_status = '{} {}'.format(response.status_code, response.reason)
@@ -927,10 +942,11 @@ class Automation:
             return response_status, pandas.DataFrame()
 
         df = pandas.read_json(response.text)
+        df.columns = _column_name_cap(df.columns)
         return df
 
     @staticmethod
-    def list_data_feed_history(api_inputs: ApiInputs, data_feed_id, top_count=10):
+    def list_data_feed_history(api_inputs: ApiInputs, data_feed_id: uuid.UUID, top_count: int = 10):
         """Retrieve data feed history
 
         Retrieves the `top_count` records for the given `data_feed_id`.
@@ -940,9 +956,9 @@ class Automation:
         api_inputs : ApiInputs
             Object returned by initialize.initialize() function
         data_feed_id : uuid.UUID
-                The unique identifier for the data feed that history should be retrieved for.
+            The unique identifier for the data feed that history should be retrieved for.
         top_count : int, default = 10
-                The top record count to be retrieved. (Default value = 10).
+            The top record count to be retrieved. (Default value = 10).
 
         Returns
         -------
@@ -950,6 +966,11 @@ class Automation:
             Dataframe containing the `top_count` history records for the given `data_feed_id`.
 
         """
+
+        if data_feed_id is None or data_feed_id == '':
+            logger.error('The data_feed_id parameter must be provided. ')
+            return
+
         headers = {
             'x-functions-key': api_inputs.api_key,
             'Content-Type': 'application/json; charset=utf-8',
@@ -957,7 +978,7 @@ class Automation:
         }
 
         url = api_prefix + api_inputs.datacentre + "/" + api_inputs.api_project_id + \
-            "/Automation/ListDataFeedsHistory/" + data_feed_id + "/" + str(top_count)
+              "/Automation/ListDataFeedsHistory/" + data_feed_id + "/" + str(top_count)
         response = requests.request("GET", url, timeout=20, headers=headers)
 
         response_status = '{} {}'.format(response.status_code, response.reason)
@@ -970,10 +991,97 @@ class Automation:
             return response_status, pandas.DataFrame()
 
         df = pandas.read_json(response.text)
+        df.columns = _column_name_cap(df.columns)
+
         return df
 
     @staticmethod
-    def get_task_code(task):
+    def data_feed_history_process_output(api_inputs: ApiInputs, data_feed_id: uuid.UUID = None,
+                                         data_feed_file_status_id: uuid.UUID = None, row_number: int = None):
+        """Retrieve data feed history process output
+
+        Retrieves the `top_count` records for the given `data_feed_id`.
+
+        Parameters
+        ----------
+        api_inputs : ApiInputs
+            Object returned by initialize.initialize() function
+        data_feed_id : uuid.UUID, default = None
+            The unique identifier for the data feed that output should be retrieved for. This parameter works with the
+            `row_number`
+        data_feed_file_status_id : uuid.UUID, default = None
+            The unique identifier for the data feed history should be retrieved for. This UUID can be retrieved from
+            the list_data_feed_history() method.
+        row_number: int, default = 0
+            The row number from the list_data_feed_history method. It can be used in place of the file_status_id. Use
+            row_number=0 to retrieve the most recent process.
+
+        Returns
+        -------
+        log : str
+            Containing the full print and output log for the process
+
+        """
+
+        if data_feed_id is None and data_feed_file_status_id is None:
+            logger.error('Must supply either the data_feed_id + row_number, or the file_status_id')
+            return
+        elif data_feed_id is not None and data_feed_file_status_id is None and row_number is None:
+            logger.error('When supplying the data_feed_id, you must also provide the row_number to retrieve. ')
+            return
+        elif data_feed_id is None and data_feed_file_status_id is not None and row_number is not None:
+            logger.error('Please supply either the data_feed_id + row_number, or the file_status_id. ')
+            return
+        elif data_feed_id is not None and data_feed_file_status_id is not None and row_number is not None:
+            logger.error('Please supply either the data_feed_id + row_number, or the file_status_id. ')
+            return
+        elif data_feed_id is not None and data_feed_file_status_id is not None and row_number is None:
+            logger.error('Please supply either the data_feed_id + row_number or the file_status_id. ')
+            return
+
+        if row_number is None:
+            row_number = -1
+
+        if data_feed_id is None:
+            data_feed_id = '00000000-0000-0000-0000-000000000000'
+
+        if data_feed_file_status_id is None:
+            data_feed_file_status_id = '00000000-0000-0000-0000-000000000000'
+
+        headers = {
+            'x-functions-key': api_inputs.api_key,
+            'Content-Type': 'application/json; charset=utf-8',
+            'user-key': api_inputs.user_id
+        }
+
+        url = api_prefix + api_inputs.datacentre + "/" + api_inputs.api_project_id + \
+              "/Automation/DataFeedsProcessOutput/" + data_feed_id + "/" + data_feed_file_status_id + "/" + \
+              str(row_number)
+
+        response = requests.request("GET", url, timeout=20, headers=headers)
+
+        response_status = '{} {}'.format(response.status_code, response.reason)
+        if response.status_code != 200:
+            logger.error("API Call was not successful. Response Status: %s. Reason: %s.", response.status_code,
+                         response.reason)
+            return response_status, pandas.DataFrame()
+        elif len(response.text) == 0:
+            logger.error('No data returned for this API call. %s', response.request.__dict__)
+            return response_status, pandas.DataFrame()
+
+        data_feed_file_process_output = json.loads(response.text, object_hook=lambda d: DataFeedFileProcessOutput(
+            data_feed_id=d['dataFeedId'], data_feed_file_status_id=d['fileStatusId'],
+            client_tracking_id=d['clientTrackingId'],
+            source_type=d['sourceType'], file_name=d['fileName'], file_received=d['fileReceived'],
+            file_process_status=d['fileProcessStatus'], file_process_status_change=d['fileProcessStatusChange'],
+            process_started=d['processStarted'], process_completed=d['processCompleted'],
+            minutes_since_received=d['minutesSinceReceived'], minutes_since_processed=d['minutesSinceProcessed'],
+            file_size=d['fileSize'], log_file_path=d['logFile'], output=d['output'], error=d['error']))
+
+        return data_feed_file_process_output
+
+    @staticmethod
+    def _get_task_code(task):
         """
 
         Parameters
